/**
 * Project mosaic
 *
 * SimControlDialog.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.util.*;
import mosaic.sim.kernel.variable.*;

// =================================================================================================
/**
* Class realizing the simulation panel which is used for controlling the simulation.
* Pops up with MenuItem "Simulation Dialog ..."
* Has buttons Reset, Start, Pause, Stop, Debug.
* Calls respective methods in the simulator.
*
* Not used currently but integrated into SimMainFrame
* 
* Changes:
* 25. May 1999 Thomas Hilpold Swing Version
*               Slider added to control speed
*               Simulation modes changed to support Animation
*
*/
public class SimControlDialog extends JDialog
  implements VariableChangeListener, ActionListener
{

  SimMainFrame parentSimFrame;

//constructor: -------------------------------------------------------------------------------------
  public SimControlDialog(SimMainFrame parent) {
    super(parent,  "Simulation Control", false);
    parentSimFrame = parent;
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    getContentPane().setLayout(new GridLayout(2,1,2,0));
    addLabelPanels();
    addButtonPanels();
    parentSimFrame.getSimApplication().getSimulator().addNrEventsListener(this);
    updateDisplay();
  }


//panels: ------------------------------------------------------------------------------------------
  private JLabel timeLabel;private JLabel eventLabel;private JLabel statusLabel;
  private void addLabelPanels() {
    this.setLocation(300, 200); // by Alice
    this.setSize(300, 80);      // by Alice
    JPanel labelPanel= new JPanel();
    labelPanel.setLayout(new FlowLayout());
    timeLabel=  new JLabel("0     ",JLabel.LEFT);
    eventLabel= new JLabel("0     ",JLabel.LEFT);
    statusLabel=  new JLabel("idle",JLabel.RIGHT);
    labelPanel.add(new JLabel("time:",JLabel.RIGHT));
    labelPanel.add(timeLabel);
    labelPanel.add(new JLabel("#events:",JLabel.RIGHT));
    labelPanel.add(eventLabel);
    labelPanel.add(new JLabel("status:",JLabel.RIGHT));
    labelPanel.add(statusLabel);

    getContentPane().add(labelPanel);
  }

//buttons: -----------------------------------------------------------------------------------------
    private JButton
    startButton;private JButton togglePauseButton;private JButton stopButton;private JButton stepButton;private JButton resetButton;private JButton debugButton;


//add GUI elements: --------------------------------------------------------------------------------
  private void addButtonPanels() {
    //create buttons and add listeners:
      startButton = new JButton("Start");
      startButton.addActionListener(this);
      togglePauseButton = new JButton("Pause");
      togglePauseButton.addActionListener(this);
      stopButton = new JButton("Stop");
      stopButton.addActionListener(this);
      stepButton = new JButton("Step");
      stepButton.addActionListener(this);
      resetButton = new JButton("Reset");
      resetButton.addActionListener(this);
      debugButton = new JButton("Debug");
      debugButton.addActionListener(this);

    //create panels and add buttons to subpanels:
      JPanel buttonPanel= new JPanel();
      buttonPanel.setLayout(new FlowLayout());
      buttonPanel.add(resetButton);
      buttonPanel.add(startButton);
      buttonPanel.add(togglePauseButton);
      buttonPanel.add(stopButton);
      buttonPanel.add(stepButton);
      buttonPanel.add(debugButton);

      getContentPane().add(buttonPanel);
    }

  // new version of update display which enables buttons  ------------------------------------------
  public void updateDisplay() {
    String timeString = new String( " " + parentSimFrame.getSimApplication().getTime() );
    timeString = timeString.substring(0, Math.min(timeString.indexOf(".")+3,timeString.length()));
    timeLabel.setText(" "  + timeString);
    eventLabel.setText(" " + parentSimFrame.getSimApplication().getSimulator().getNrEvents());
    if (parentSimFrame.getSimApplication().getSimulator().isRunning()) {
      statusLabel.setText("running");
      startButton.setEnabled (false);
      stopButton.setEnabled (true);
      stepButton.setEnabled (false);
      resetButton.setEnabled (false);
    } else {
      statusLabel.setText("     idle");
      statusLabel.setForeground(buttonActiveColor);
      startButton.setEnabled (true);
      stopButton.setEnabled (false);
      stepButton.setEnabled (true);
      resetButton.setEnabled (true);
    }
    debugButton.setEnabled (true);
    timeLabel.setForeground(numberColor);
    eventLabel.setForeground(numberColor);
    statusLabel.setForeground(statusColor);
  }

//colors: ------------------------------------------------------------------------------------------
  private Color buttonActiveColor= Color.black;
  public void setButtonActiveColor(Color c) { buttonActiveColor= c; updateDisplay(); }
  public Color getButtonActiveColor() { return buttonActiveColor; }
  private Color buttonDimmedColor= Color.red;
  public void setButtonDimmedColor(Color c) { buttonDimmedColor= c; updateDisplay(); }
  public Color getButtonDimmedColor() { return buttonDimmedColor; }
  private Color numberColor= Color.blue;
  public void setNumberColor(Color c) { numberColor= c; updateDisplay(); }
  public Color getNumberColor() { return numberColor; }
  private Color statusColor= Color.blue;
  public void setStatusColor(Color c) { statusColor= c; updateDisplay(); }
  public Color getStatusColor() { return statusColor; }
  public void setForeground(Color c) { super.setForeground(c); updateDisplay(); }


// window closing ----------------------------------------------------------------------------------

  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      setVisible(false);
    }
    super.processWindowEvent(e);
  }

// ActionListener ----------------------------------------------------------------------------------

    public void actionPerformed (ActionEvent e) {
      String cmd;
      cmd = e.getActionCommand();
      Object sourceComp = e.getSource();
      SimMainFrame mainFrame = (SimMainFrame)getParent();

      if (sourceComp == resetButton) mainFrame.doReset();
      else if (sourceComp == startButton) mainFrame.doStart();
      else if (sourceComp == stopButton) mainFrame.doStop();
      else if (sourceComp == stepButton)  mainFrame.doStep();
      else if (sourceComp == debugButton) mainFrame.doDebug();

      updateDisplay(); 
    }


// VariableChangeListener --------------------------------------------------------------------------

    /** Reacts to a variable changed event. */
    public void variableChange (VariableChangeEvent e) {
        updateDisplay();
    }

    /** Reacts to a property changed event. */
    public void propertyChange (PropertyChangeEvent e){
    }

    /** Adds a listener variable. */
    public void addListenerVariable (Variable v){
    }

    /** Removes a listener variable. */
    public void removeListenerVariable (Variable v){
    }

    /** variable to hold the priority for handling this listener in handling variable change events. */
    protected int priority;

    /**
     * Gets the priority for the handling of variable change events by the simulator (the higher
     * the priority, the earlier it will be handled.)
     */
    public final int getPriority() {
        return priority;
    }

    /**
     * Sets the priority for the handling of variable change events by the simulator (the higher
     * the priority, the earlier it will be handled.)
     */
    public final void setPriority(int priority) {
        this.priority = priority;
    }

} //================================================================================================
