package mosaic.sim.kernel.continuous;

import java.lang.*;
import java.util.*;
import java.lang.reflect.*;
import mosaic.sim.kernel.variable.DoubleVariable;

/**
*
*/

abstract class BasicIntegrationMethod
   extends Object
   implements IntegrationMethod
   {

   public BasicIntegrationMethod () {
      super();
   } // BasicIntegrationMethod


   /** Obergrenze fr die dynamisch angelegen Arrays */
   protected int nrStates;

   /** Array zum Zwischenspeichern der Zustnde fr den Zugriff auf ihre Ableitungen */
   protected DoubleVariable[] states;

   /** Array zum Zwischenspeichern der alten Zusnde (vor dem Integrationsschritt) */
   protected double[] oldStates;

   /** Array zum Zwischenspeichern der Integrationskoeffizienten */
   protected double[][] k;

   /**
   * Alias fr die bergebenen Modelle (damit nicht immer ber <code>data</code>
   * zugegriffen werden mu.
   */
   protected ContEquation[] equations;

   /*
   * Initialisiert die Klassenvariablen aufgrund der an die Integrationsmethode
   * bergebenen Daten.
   * @param data an die Integrationsmethode bergebenen Daten
   */
   public void initialize(ArrayList stateArrayList, ArrayList equationsArrayList) {
      nrStates = stateArrayList.size();
      oldStates = (double[])Array.newInstance(Double.TYPE, nrStates);
      states = (DoubleVariable[])Array.newInstance(new DoubleVariable().getClass(), nrStates);

      // make state array
      for(int i=0;i < nrStates;i++)
         states[i] = (DoubleVariable)stateArrayList.get(i);

      int nrEquations = equationsArrayList.size();
      equations = new ContEquation[nrEquations];
      // make equation array
      for(int i=0;i < nrEquations;i++)
         equations[i] = (ContEquation)equationsArrayList.get(i);
      }

   /**
   * Fhrt die Geleichungen der einzelnen Untermodelle aus.
   */
   public void doEquations()
      throws ContSimulationException {
      for(int i=0;i < equations.length;i++)
         equations[i].equation();
   }
}


// end of BasicIntegrationMethod.java
