package mosaic.sim.kernel.control;
import java.util.*;

/**
 *
 * Synchronization between Animators and between Animators and getSimApplication().getSimulator() via real time clock
 * with taking the AnimationRate into Account
 *
 */

public class Animator {

    //
    // DEBUG output
    // if (DEBUG[n]) ...
    private static boolean[] DEBUG = new boolean[] {
        false,  //
        false   //
        };

    /** keeps track of the views that already painted in a frame */
    protected volatile int curFramePaintedViews;
    protected Vector views;
    Thread schedulerThread;

    //
    // Constructors
    //

    public  Animator() {
        views = new Vector(10,10);
    }


    int nrOfRepaintingViews = 0;
    boolean animated = false;
    boolean stopAsSoonAsPossible = false;

    /** Calls performAnimation on all registeredViews */
    void animate(double simTime) {
        //ensure that we are past the simTime
        schedulerThread = Thread.currentThread();
        nrOfRepaintingViews = 0;
        curFramePaintedViews = 0;
        animated = false;
        for (int i = 0; i < views.size(); i++) {
            if (((AnimatedView)views.elementAt(i)).performAnimationStep(simTime)) {
                nrOfRepaintingViews++;
                //System.out.println(" time = " + getSimRealTime());
            }
        }
        animated = true;
        /*
        while (!allViewsPainted()) {
             try {
                // System.out.print("s");
                 Thread.currentThread().sleep(100000);
             } catch (InterruptedException e) {
                //System.out.print(".");
             };
        }
        */
    }

    /** Each view muist call this method after paint was called due to an animation step */
    // Waht if it doesnt get painted because it is not shown ??????????
    public void nextViewPainted() {
        curFramePaintedViews++;
        /*
        if ((schedulerThread != null) && (allViewsPainted())) {
                //tell the Animater to wake up and continue animation
              schedulerThread.interrupt();
        }
        */

    }

    private boolean allViewsPainted() {
        //if (((animated) && (curFramePaintedViews >= nrOfRepaintingViews))) System.out.print("\n anim = " + animated + "curF = " + curFramePaintedViews + " nrShould " + nrOfRepaintingViews);
        return ((animated) && (curFramePaintedViews >= nrOfRepaintingViews));
    }

    /** Registers a view */
    public void addView(AnimatedView newView) {
        if (!views.contains(newView)) {
            views.add(newView);
        }
    }

    /** removes a view, so it wont be called to perform animation steps any more */
    public void removeView(AnimatedView view) {
        views.remove(view);
    }
}

