//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class NumberInputView -------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.input;

import javax.swing.*;

/**
* Animation view to visualize and allow input of numeric values.
* Can serve as variable change listeners for numeric variables.
* Can serve as userInputSource for numeric values
* Displays the number in a textfield.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/

public class BasicNumberInputView extends VariableInputView {

  /** Default constructor */
  public BasicNumberInputView() {
    super();
        try  {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
  }

// number value ----------------------------------------------------------------------------------------------

  /** variable to store the number object */
  Number number;

  /** Sets the number object received by a variableChange */
  public void setNumber(Number newNumber) {
    number= newNumber;
  }
  /** Gets the number object */
  public Number getNumber() { return number; }

// setValue --------------------------------------------------------------------------------------------------

  /**
  * Sets the value object received by a variableChange.
  * Specialized from ValueView.
  * @param newVal   new value object received from variableChange.
  */
  public void setValue (Object newVal) {
        if (newVal instanceof Number) {
            super.setValue(newVal);
            setNumber ((Number)newVal);
        }
        else
            System.out.println(">> tried to send non Number value to " + toString());
     }

    private void jbInit() throws Exception {
    }
}
