//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class BoundNumberInputView    -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.input;

/**
* Animation view to visualize and allow input of numeric values.
* Can serve as variable change listeners for numeric variables.
* Can serve as userInputSource for numeric values
* Displays the number in a textfield.
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* <li> set minimum input value by <code>setMinVal</code>
* <li> set maximum input value by <code>setMaxVal</code>
** </ul>
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/

public class BoundNumberInputView extends NumberInputView {

  /** Default constructor */
  public BoundNumberInputView() {
    super();
  }

  /** reads a numeric value from text and returns it as Object */
  public Object readValue(String text) {
    try
    { Double value = new Double(text);
      if (value.doubleValue() < minVal)
        return new Double(minVal);
      else if (value.doubleValue() > maxVal)
        return new Double(maxVal);
      else
        return value;
    }catch (NumberFormatException e)
    { return null;
    }
  }

// setValue --------------------------------------------------------------------------------------------------

  /**
  * Sets the value object received by a variableChange.
  * Bounds received value to min and max.
  * Specialized from ValueView.
  * @param newVal   new value object received from variableChange.
  */
  public void setValue (Object newVal) {
        if (newVal instanceof Number) {
          if ( ((Number)newVal).doubleValue() < minVal) {
              setNumber(new Double(minVal));
              super.setValue(new Double(minVal));
          }
          else if ( ((Number)newVal).doubleValue() > maxVal) {
            setNumber( new Double(maxVal));
            super.setValue(new Double(maxVal) );
          }
          else {
            setNumber ((Number)newVal);
            super.setValue(newVal);
          }
        }
        else
            System.out.println(">> tried to send non Number value to " + toString());
     }


// setMinVal -------------------------------------------------------------------
  double minVal = 0.0;

  /**
  * Sets the new minimum for values entered, if actual stored value is lower
  * than this, it will be corrected.
  */
  public void setMinVal(double newMin) {
    minVal = newMin;
    if (getNumber() != null)
      if (getNumber().doubleValue() < minVal)
        setValue(new Double(minVal));
  }

  /**
  * Gets the actual minimum for values entered.
  */
  public double getMinVal() {
    return minVal;
  }

// setMaxVal -------------------------------------------------------------------
  double maxVal = 100.0;

  /**
  * Sets the new maximum for values entered, if actual stored value is bigger
  * than this, it will be corrected.
  */
  public void setMaxVal(double newMax) {
    maxVal = newMax;
    if (getNumber() != null)
      if (getNumber().doubleValue() > maxVal)
        setValue(new Double(maxVal));
  }

  /**
  * Gets the actual minimum for values entered.
  */
  public double getMaxVal() {
    return maxVal;
  }

}
