
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class CollectionTextView     ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.output;

import java.awt.*;
import java.util.*;

/**
* Animation view to visualize a vector of Objects.
* <p>
* Use the view by adding it as VariableChangeListener to a <code>CollectionVariable</code>.
* Displays the textual representation of the Objects using toString.  
* <p>
* <b>Costumization: </b>
* <ul>
* <li> set the label of the view by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
*
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/
public class CollectionTextView extends TextView {

  /** Default constructor QueueView */
  public CollectionTextView() {
  }

  /**
  * Reacts to a new property value by setting the Collection value.
  * Sets the text by calling <code>toString</code> of the vector.
  * Called by super.propertyChange.
  * @param newVal The new vector object received from the property change
  */
  public void setValue (Object newVal) {
      if ((newVal instanceof Collection) || (newVal == null)) {
          super.setValue(newVal);
          String htext = "";
          if (getValue() != null) {
              Collection v = (Collection)getValue();
              Iterator i = v.iterator();
              while (i.hasNext()) {
                htext = htext + i.next().toString();
              }
          setText ( htext );
          }
      }
      else
            System.out.println(">> tried to send non Collection value to " + toString());
  }

    /**
    * Paints the object;
    * by giving out its textual representation.
    *
    * @param g the graphics context of the view
    */
    public void paintValue(Graphics g) {
        int width = getSize().width;
        int height = getSize().height;
        g.setColor(getForeground());
        g.setFont(getFont());
        FontMetrics fm = g.getFontMetrics();
        Collection v = (Collection)getValue();
        g.drawString(getText(), 2, 14);
    }

}

