
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class ComponentPaintView       ---------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.output;

import java.awt.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.output.*;
import javax.swing.*;

/**
* Animation view to paint <code>Component</code> values. 
* Paints the <code>Component</code> in the view. 
* <p>
* Use the view by adding it as VariableChangeListener to an ObjectVariable. 
* <p>
* Costumization: 
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* </ul>
* 
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G% 
*/
public class ComponentPaintView extends VariableView {

  /** Default constructor for PaintComponentView */
  public ComponentPaintView() {
      setLayout (new FlowLayout()); 
  }

   /** Sets the <code>Component</code> value and adds it to the container. */
  public void setValue (Object newVal) {
      super.setValue(newVal);
      removeAll();
      if (getValue() != null) {
        if (newVal instanceof Component) {
          add((Component)newVal);
        }
        else  {
          System.out.println(">> tried to send non JComponent value to " + toString());
        }
      }
      validate();
      repaint();
  }

  /** Empty  because painting of item is done by Container.paint */
  public void paintValue(Graphics g) {
  }
}

