
//Project:      SimBeans
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class ConstantTimeSeed           -----------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 29.12.98
// documented by: hp, 29.12.98

package mosaic.sim.kernel.random;

import mosaic.sim.kernel.*;
/**
* Class realizing a TimeSeed which provides constant time intervals.
* This class e.g. is used for scheduling events at constant time intervals.
*
* <b>Customization </b>
* <ul>
* <li>Use setConstant to set the constant for the time intervals.  
* </ul>
*/
public class ConstantTimeSeed extends BasicRandomComponent
                                implements TimeSeed  {


  /** Constructor with constant as argument */
  public ConstantTimeSeed (double iConstant) {
      setConstant (iConstant);
  }

  /** Constructor with no argument. Constant is set to 1.0 */
  public ConstantTimeSeed () {
      setConstant (1.0);
  }

  /** Gets next time span (to the next event by calling getRandDouble*/
  public double getTimeSpan() {
      return constant;
  }

  /** Sets the constant for the time intervals.*/
  public void setConstant(double newConstant) {
      constant = newConstant;
  }

  /** Gets the constant for the time intervals. */
  public double getConstant() {
      return constant;
  }
  
  private double constant = 1.0;
}
