
//Project:        mosaic kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// interface SimEventComponent -------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 14.12.98
// documented by: hp, 14.12.98


package mosaic.sim.kernel.simevent;

import mosaic.sim.kernel.ModelComponent;

/**
* Interface which must be supported by all simulation components which fire SimEvents.
* All components of type SimEventComponent have a
* <ul>
* <li> property tL (time of last event) to store the time the last SimEvent occurred in this component
* <li> property selectOrder to specify the priority of this component in tie break with other component in the case of simultaneous events.
* <li> method triggerEvent which is used by the Simulator to trigger the execution of the next event in this component.
* <li> method trace which is used by the Simulator to put out trace information in SimEventTraceWindows.
* </ul>
*/
public interface SimEventComponent extends ModelComponent {

  /** Gets the time of last event. */
  public double getTL();
  /** Sets the time of last event. */
  public void setTL(double newTime);

  /** Gets the select order. */
  public int getSelectOrder();
  /** sets the select order. */
  public void setSelectOrder(int newOrder);

  /** Trigger an event. Used by the Simulator.  */
  public void triggerEvent();

  /** Traces an event in an open trace window.  */
  // public void trace();

// SimEvent -------------------------------------------------------------------------------------------------

  /** Adds a SimEventListener */
    public void addSimEventListener (mosaic.sim.kernel.simevent.SimEventListener l);

  /** Removes a SimEventListener */
    public void removeSimEventListener (mosaic.sim.kernel.simevent.SimEventListener l);

}
