
//Project:        mosaic kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// interface SimEventListener  -------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------
// programmed by: hp
// last modified by: hp, 14.12.98
// documented by: hp, 14.12.98


package mosaic.sim.kernel.simevent;

import java.util.EventListener;
import java.io.Serializable;

/**
* Interface for listeners to SimEvents.
* SimEventListeners are typically simulation model classes which react to fundamental simulation events,
* like time scheduled events from Timers, state events in combined simulation or imput events from the
* user.
* <p>
* Specifies method <code>processSimEvent (SimEvent evt)</code> as the event method to react to SimEvents. 
*/
public interface SimEventListener extends EventListener, Serializable  {

   /** Reacts to SimEvents from SimEventComponents. 
   * This is the event method for reacting to SimEvents
   */
  public void processSimEvent (mosaic.sim.kernel.simevent.SimEvent evt);

  /** Adds a SimEventComponent this listener listens to. Is called from addSimEventListener of SimEventComponent. */
    public void addSimEventComponent (mosaic.sim.kernel.simevent.SimEventComponent c);

  /** Removes a SimEventComponent this listener listens to. Is called from removeSimEventListener of SimEventComponent.  */
    public void removeSimEventComponent (mosaic.sim.kernel.simevent.SimEventComponent c);

}
