
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// interface TimeDependentStatistic    -----------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.sim.kernel.statistics;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.control.*;

/**
* Class for time weighted statistics of number values.
* Computes max, min, mean and std devition of the values received per time.
* <p>
* Use this class for computing min, max, mean and std devition of the values of numeric properties
* per time unit (is time weighted, e.g. mean queue length or utilization per time unit).
* <p>
* <b>Bound properties:</b>
* <dl>
* <dt><code> min      </code></dt> <dd> the minimum value received </dd>
* <dt><code> max      </code></dt> <dd> the maximum value received </dd>
* <dt><code> mean     </code></dt> <dd> the mean of the values received </dd>
* <dt><code> stdDev   </code></dt> <dd> the std deviaion of  values received </dd>
* </dl>
*
* @see   ObservationalStatistic
*
* @author Herbert Praehofer
* @version %I% %G%
*/
public class TimeDependentStatistic extends BasicStatistic
{
  /** Default constructor VariableStatistic */
  public TimeDependentStatistic() {
  }

// -----------------------------------------------------------------------------------------------------------
// statistic values ------------------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------


// old value -------------------------------------------------------------------------------------------------

  protected double val = 0;

  protected void setVal (double newVal) {
      val = newVal;
  }

  public double getVal () {
      return val;
  }

// mean   ----------------------------------------------------------------------------------------------------

  /** variable to hold the mean value */
  double mean;

  /** Gets the mean value */
  public double getMean () {
     return mean;
  }

  /** Sets the minimum value */
  protected void setMean (double newMean) {
     mean = newMean;
        // ---> add signaling property change here
  }



// standard deviation    -------------------------------------------------------------------------------------

  /** variable to hold the stdDeviation value */
  double stdDev;

  /** Gets the stdDev value */
  public double getStandardDeviation () {
        return stdDev;
  }

  /** Sets the stdDev value */
  protected void setStandardDeviation (double newStdDev) {
        stdDev = newStdDev;
        // ---> add signaling property change here
  }

 
// help variables --------------------------------------------------------------------------------------------

  /** variable to hold the area of the values (= values * time)  */
  double area;

  /** Gets the area value */
  public double getArea () {
        return area;
  }

  /** Sets the area value */
  protected void setArea (double newArea) {
        area = newArea;
  }

  /** variable to hold the area of the values (= values * time)  */
  double areaOfSquares;

  /** Gets the areaOfSquares value */
  public double getAreaOfSquares () {
        return areaOfSquares;
  }

  /** Sets the areaOfSquares value */
  protected void setAreaOfSquares (double newAreaOfSquares) {
        areaOfSquares = newAreaOfSquares;
  }


// compute statistics ----------------------------------------------------------------------------------------

  /** Computes the statistics based on the received number values */
  protected void computeStatistics() {
    super.computeStatistics ();
    double elapsedT = getTime() - getTL();
    setArea (getArea() + (val*elapsedT));
    setAreaOfSquares (getAreaOfSquares() + (val*val*totalElapsedT));
    if (getTotalElapsedT() != 0) {
        setMean(getArea() / totalElapsedT);
        //setStandardDeviation (Math.sqrt(Math.abs(getAreaOfSquares() - (mean* mean * totalElapsedT)) / totalElapsedT));
    }
    setVal(number.doubleValue());
    setTL (getTime());
  }



// reset -----------------------------------------------------------------------------------------------------

  /** Clears the statistics */
  protected void clearStatistics() {
    super.clearStatistics();
    setArea(0);
    setAreaOfSquares(0);
    setMean (0);
    setStandardDeviation (0);
    setVal(0);
  }

}

