
//Project:      mosaic
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria
/*
 *  AbstractVariableChangeListener
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.util.ArrayList;
import java.beans.PropertyChangeEvent;
import mosaic.sim.kernel.AbstractModelComponent;

abstract public class AbstractVariableChangeListener extends AbstractModelComponent
                                            implements VariableChangeListener {


// constructors ------------------------------------------------------------------------------------

    /** Default constructor */
    public AbstractVariableChangeListener() {
        this("VariableChangeListener");
    }


    /**  Constructor with parent model and label. */
    public AbstractVariableChangeListener( String label) {
        super(label);
    }


// variable and property change --------------------------------------------------------------------

    /** Reacts to a variable changed event. */
    abstract public void variableChange (VariableChangeEvent e);

    /** Reacts to a variable changed event. */
    public void propertyChange (PropertyChangeEvent e) {
    }

// listener variables ------------------------------------------------------------------------------

    /** List to hold the variables this listener listens to currently */
    protected ArrayList listenerVariables = new ArrayList();

    /** Adds a listener variable. */
    public void addListenerVariable (Variable v){
        listenerVariables.add(v);
    }

    /** Removes a listener variable. */
    public void removeListenerVariable (Variable v) {
        listenerVariables.remove(v); 
    }

// priority ----------------------------------------------------------------------------------------

    /** variable to hold the priority for handling this listener in handling variable change events. */
    protected int priority = 1;

    /**
     * Gets the priority for the handling of variable change events by the simulator (the higher
     * the priority, the earlier it will be handled.)
     */
    public final int getPriority() {
        return priority;
    }

    /**
     * Sets the priority for the handling of variable change events by the simulator (the higher
     * the priority, the earlier it will be handled.)
     */
    public final void setPriority(int priority) {
        this.priority = priority;
    }


} // end AbstractVariableChangeListener ============================================================
