/*
 *  ArrayListVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.util.*;

/**
* Class ArrayListVariable is a Variable for values of type ArrayList.
* Use this bean class to realize simulation model variables of type ArrayList.
* Has a ArrayList value and <code>getValue</code> and <code>setValue</code> to read and write the vector.
* Additionally provides all common List methods to access the members of the ArrayList, e.g. add,
* remove, etc. and signals a change whenever the vector changes.
* <p>
* Additionally provides the number of elements in the ArrayList (nrElements) and allows to
* add and remove VariableChangeListeners to nrElements (addNrElementsListener, removeNrElementsListener).
*
* @see sim.kernel.Variable
*/
public class ArrayListVariable
   extends mosaic.sim.kernel.variable.ListVariable {

// Constructors ----------------------------------------------------------------------------------------------

  /** Default constructor for BooleanVariable*/
   public ArrayListVariable() {
        this("", VARIABLE, "", "", "", new ArrayList());
      }

  /** Constructor for BooleanVariable initializing model owner and value. */
   public ArrayListVariable (ArrayList value) {
        this("", VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public ArrayListVariable (String name, ArrayList value) {
        this(name, VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public ArrayListVariable (String name, int type, ArrayList value) {
        this(name, type, "", "", "", value);
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public ArrayListVariable(String name, int type,
                    String unit, String quantity, String displayUnit, ArrayList value ) {
      super(name, type, unit, quantity, displayUnit);
      this.value = value;
   }

// ArrayList value ---------------------------------------------------------------------------------

  /** field to hold the vector of the variable */
  protected ArrayList value;

  /** Gets the Object vector of the variable */
  public final ArrayList getValue () {
    return ((ArrayListVariable)definedBy).value;
  }

  /** Sets the Object vector of the variable */
  public final void setValue (ArrayList newValue) {
    if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
      getSimApplication().getIntegrator().stateEventOccurred = true;
      return;
    }
    value = newValue;
    if (definedBy==this) signalVariableChange();
  }


// getValueObject      -----------------------------------------------------------------------------

  /** gets the vector of the variable as type Object */
  public final Object getValueObject () {
      return ((ArrayListVariable)definedBy).value;
  }

// getValueAsList
    /** Gets the value as List object */
    public final List getValueAsList () {
        return ((List) value);
    }

// getValueAsCollection
    /** Gets the value as List object */
    public final Collection getValueAsCollection () {
        return ((Collection) value);
    }

}
