/*
 *  ObjectVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */

package mosaic.sim.kernel.variable;

import java.lang.*;
import mosaic.sim.kernel.*;
import mosaic.sim.kernel.simevent.*;



/**
* ObjectVariable is a Variable for values of type Object.
* Use this bean class to realize simulation model variables of type Object.
* Has a Object value field and <code>getValue</code> and <code>setValue</code> methods to read and write it. 
* Inherits the features of Variable, in particular: to signal variableChangeEvents.
*
* IMPORTANT: Always set the model owner of a variable, either in the contructor
* or by adding it to the model (usually done in interactive mode by the builder tool.)
*
* @see sim.kernel.Variable
*/
public class ObjectVariable
   extends mosaic.sim.kernel.variable.AbstractVariable
{

// Constructors ------------------------------------------------------------------------------------


  /** Default constructor for BooleanVariable*/
   public ObjectVariable() {
        this("", VARIABLE, "", "", "", null);
      }

  /** Constructor for BooleanVariable initializing model owner and value. */
   public ObjectVariable (Object value) {
        this("", VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public ObjectVariable (String name, Object value) {
        this(name, VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public ObjectVariable (String name, int type, Object value) {
        this(name, type, "", "", "", value);
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public ObjectVariable(String name, int type,
                    String unit, String quantity, String displayUnit, Object value ) {
      super(name, type, unit, quantity, displayUnit);
      this.value = value;
   }

// Object value      -------------------------------------------------------------------------------

  /** field to hold the value of the variable */
    protected Object value;

    /** Gets the Object value of the variable */
    public Object getValue () {
      return ((ObjectVariable)definedBy).value;
    }

    /** Sets the Object value of the variable */
    public final void setValue (Object newValue) {
      if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
        getSimApplication().getIntegrator().stateEventOccurred = true;
        return;
      }
      value = newValue;
      if(definedBy==this) signalVariableChange();
    }


// getValueObject      -----------------------------------------------------------------------------

  /** gets the value of the variable as type Object */
    public Object getValueObject () {
      return ((ObjectVariable)definedBy).value;
    }


// overwrite method processSimEvent from interface InputEventListener  -----------------------------

  /** Reacts to SimEvents from SimEventComponents.
   *  The events should be triggered by a inputEventComponent and
   *  return Number-Objects.
   */
  public void processSimEvent (SimEvent evt){
  Object obj;
    if ((evt.getSource()) instanceof InputEventComponent){
      obj = ((InputEventComponent)(evt.getSource())).getInput();
      if (obj instanceof Integer)
        this.setValue( obj );
    }
  }


}

