/**
 * Project mosaic
 *
 * VariableChangeListener.java
 *
 * copyright 2000, JKU Linz, Austria
 *
 */

package mosaic.sim.kernel.variable;

import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.beans.PropertyChangeEvent;

/**
* Interface for listeners of VariableChangeEvents.
* Specifies variableChange event method and methods for adding and removing Variables.
* Extends java.beans.PropertyChangeListener
*/
public interface VariableChangeListener
    extends PropertyChangeListener, Serializable  {

  /** Reacts to a variable changed event. */
  public void variableChange (mosaic.sim.kernel.variable.VariableChangeEvent e);

  /** Reacts to a property changed event. */
  public void propertyChange (PropertyChangeEvent e);

  /** Adds a listener variable. */
  public void addListenerVariable (mosaic.sim.kernel.variable.Variable v);

  /** Removes a listener variable. */
  public void removeListenerVariable (mosaic.sim.kernel.variable.Variable v);

  /**
   * Gives a priority for the handling of variable change events by the simulator (the higher
   * the priority, the earlier it will be handled.)
   */
  public int getPriority();

} // end VariableChangeListener --------------------------------------------------------------------
