/**
 * Project mosaic
 *
 * VariableChangeRegistry.java
 *
 * copyright 2000, JKU Linz, Austria
 */

package mosaic.sim.kernel.variable;

import java.lang.*;
import java.lang.reflect.*;
import java.beans.*;
import mosaic.sim.kernel.*; 
/**
* Class for registration of variable change events.
* <p>
* For a variable change event to a model object an VariableChangeRegistry is created and queued
* gobally in the simulator.
* The VariableChangeRegistries are handled in fifo manner by the simulator by
* accessing the variable value and sending an VariableChangeEvent to the listener l.
*/

public class VariableChangeRegistry extends ChangeRegistry {

// Constructors ------------------------------------------------------------------------------------

    /** 
    * Constructor for the VariableChangeRegistry initializing model source, variable, and the listener.
    * Gets the get method from the name through the reflection package. 
    */
    public VariableChangeRegistry (mosaic.sim.kernel.variable.Variable v, mosaic.sim.kernel.variable.VariableChangeListener l) {
        super((ModelComponent)v);
        variable = v;
        listener = l;
    }


// variable ----------------------------------------------------------------------------------------

    /** variable to hold the variable of the variable change event */
    mosaic.sim.kernel.variable.Variable variable;

    /** Gets the variable of the event */
    public mosaic.sim.kernel.variable.Variable getVariable() {
        return variable;
    }


// listener ------------------------------------------------------------------------------------------

    /** variable to hold the listener for the variable change event */
    mosaic.sim.kernel.variable.VariableChangeListener listener;

    /** Gets the listener of the event */
    public mosaic.sim.kernel.variable.VariableChangeListener getListener() {
        return listener;
    }

// fireChangeEvent ---------------------------------------------------------------------------------

    /**
    * Executes a variable change event.
    * Accesses the variable value by the getValueObject method,
    * sets the value of the VariableChangeEvent
    * and sends a variableChange event to the VariableChangeListener.
    */
    public final void fireChangeEvent () {
       listener.variableChange(
            new VariableChangeEvent(variable, variable.getValueObject(), variable.getChangedTime()));
    }

// toString ----------------------------------------------------------------------------------------

    public String toString() {
        String output = "VariableChange: ";
        try {
            output = output + " variable "  + variable.getLabel();
            Object value = variable.getValueObject();
            if (value == null) {
                output = output + " value null ";
            } else {
                output = output + " value "  + variable.getValueObject().toString();
            }
            ModelComponent sc = (ModelComponent) listener;
            output = output + " to listener "  + sc.getLabel();
        }
        catch (Exception e) { }
        return output;
    }

} // end VariableChangeRegistry --------------------------------------------------------------------

