/*
 *  VectorVariable
 *
 * @version 2.0 alpha
 * @date
 *
 * copyright: JKU Linz
 */


package mosaic.sim.kernel.variable;

import java.util.*;

/**
* Class VectorVariable is a Variable for values of type Vector.
* Use this bean class to realize simulation model variables of type Vector.
* Has a Vector value and <code>getValue</code> and <code>setValue</code> to read and write the vector.
* Additionally provides all common List methods to access the members of the Vector, e.g. add,
* remove, etc. and signals a change whenever the vector changes.
* <p>
* Additionally provides the number of elements in the Vector (nrElements) and allows to
* add and remove VariableChangeListeners to nrElements (addNrElementsListener, removeNrElementsListener).
*
* @see sim.kernel.Variable
*/
public class VectorVariable extends ListVariable {

// Constructors ----------------------------------------------------------------------------------------------

  /** Default constructor for BooleanVariable*/
   public VectorVariable() {
        this("", VARIABLE, "", "", "", new Vector());
      }

  /** Constructor for BooleanVariable initializing model owner and value. */
   public VectorVariable (Vector value) {
        this("", VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, and value. */
   public VectorVariable (String name, Vector value) {
        this(name, VARIABLE, "", "", "", value);
      }

  /** Constructor for BooleanVariable initializing model owner, name, variable type, and value. */
   public VectorVariable (String name, int type, Vector value) {
        this(name, type, "", "", "", value);
     }

  /** Constructor for BooleanVariable initializing model owner, name,
  * description, unit of measurement, quantity, displayUnit, variable type, and value. */
  public VectorVariable(String name, int type,
                    String unit, String quantity, String displayUnit, Vector value ) {
      super(name, type, unit, quantity, displayUnit);
      this.value = value;
   }

// Vector value ---------------------------------------------------------------------------------

  /** field to hold the vector of the variable */
  protected Vector value;

  /** Gets the Object vector of the variable */
  public final Vector getValue () {
    return ((VectorVariable)definedBy).value;
  }

  /** Sets the Object vector of the variable */
  public final void setValue (Vector newValue) {
    if (getSimApplication() != null && getSimApplication().getIntegrator().testForStateEvents) {
      getSimApplication().getIntegrator().stateEventOccurred = true;
      return;
    }
    value = newValue;
    if (definedBy==this) signalVariableChange();
  }


// getValueObject      -----------------------------------------------------------------------------

  /** gets the vector of the variable as type Object */
  public final Object getValueObject () {
      return ((VectorVariable)definedBy).value;
  }

// getValueAsList
    /** Gets the value as List object */
    public final List getValueAsList () {
        return ((List) value);
    }

// getValueAsCollection
    /** Gets the value as List object */
    public final Collection getValueAsCollection () {
        return ((Collection) value);
    }

}
