/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: ActiveVariable.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.6 $
 *  Last changed at: $Date: 2002/05/19 17:20:27 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron;

import mosaic.sim.kernel.simevent.*;
import mosaic.sim.kernel.variable.*;

/** This class implements an active variable that has support for sending out
 *  VariableChangeEvents and for sending itself a TimeEvent, therefore being an
 *  active variable container.
 */
public abstract class ActiveVariable extends AbstractVariable
							implements TimeEventListener {
	protected Timer timer;

	/** Creates a new timer object and registers this object as a listener for
	 *  time events.
	 *  @see #timer
	 */
	public ActiveVariable() {
		timer = new Timer();
		timer.addTimeEventListener(this);
	}

	/** Creates a new timer object and registers this object as a listener for
	 *  time events.
	 *  @param name The name of the timer object (for event list outputs).
	 *  @see #timer
	 */
	public ActiveVariable(String name) {
		timer = new Timer(name);
		timer.addTimeEventListener(this);
	}

	/** This implementation of TimeEventListener.addTimeEventComponent does
	 *  nothing */
	public void addTimeEventComponent (mosaic.sim.kernel.simevent.SimEventComponent c) {
	}

	/** This implementation of TimeEventListener.addTimeEventComponent does
	 *  nothing */
	public void removeTimeEventComponent (mosaic.sim.kernel.simevent.SimEventComponent c) {
	}

	/** Calls the respective method in timer
	 *  @see #timer
	 */
	public void activateAt (double time) {
		timer.activateAt(time);
	}

	/** Calls the respective method in timer
	 *  @see #timer
	 */
	public void activateIn (double deltaT) {
		timer.activateIn(deltaT);
	}

	/** Calls the respective method in timer
	 *  @see #timer
	 */
	public void passivate() {
		timer.passivate();
	}

	/** Calls the respective method in timer
	 *  @see #timer
	 */
	public boolean isActive() {
		return timer.isActive();
	}
}
