/*****************************************************************************
 *  Project: DEVSNeuron
 *  Description: discrete event-based simulation of spiking neural networks
 *  Filename: $RCSfile: ViewSynchronizer.java,v $
 *  Author: Rene Mayrhofer
 *  Copyright: Rene Mayrhofer 2001
 *  Project version tag: $Name:  $
 *  File version: $Revision: 1.4 $
 *  Last changed at: $Date: 2002/05/16 20:43:35 $
 *  Last changed by: $Author: rene $
 *****************************************************************************/

package mosaic.sim.neuron.visualisation;

import java.util.*;
import javax.swing.JComponent;

/** This class is able to synchronize multiple ScrollingViews so that they
 *  always show the same current simulation time.
 */
public class ViewSynchronizer extends JComponent {

    /** The list of views that should be synchronized. */
	protected ArrayList views = new ArrayList();

	/** Creates a ViewSynchronizer. */
	public ViewSynchronizer() {
	}

	/** Adds a view that should be synchronized with the already added views.
	 *  @see #views
	 */
	public void addView(ScrollingView view) {
		views.add(view);
		view.setSynchronizer(this);
	}

	/** Removes a view that should no longer be synchronized with the already
	 *  added views.
	 *  @see #views
	 */
	public boolean removeView(ScrollingView view) {
		if (views.remove(view)) {
			view.setSynchronizer(null);
			return true;
		}
		else
			return false;
	}

	/** Scrolls all registered ScrollingViews horizontally so that they show
	 *  the same current simulation time.
	 *  @param currentTime The simulation time to be shown in all views.
	 */
	public void scrollToSimulationTime(double currentTime) {
		for (int i=0; i<views.size(); i++)
			((ScrollingView) views.get(i)).scrollToSimulationTime(currentTime);
	}
}
