
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class TwoDimHashtable          ----------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------


package mosaic.util;

import java.util.*;

/**
* Class TwoDimHashtable implements a Hashtable which contains Hashtable as values
* which contain vectors which finally contain the elements.
* Use this class to realize a two dimensional mapping of two keys to a set of values.
*/
public class TwoDimHashtableOfVectors extends mosaic.util.TwoDimHashtable {

  public TwoDimHashtableOfVectors() {
  }
  public TwoDimHashtableOfVectors(int i) {
      super(i);
  }
  public TwoDimHashtableOfVectors(int i, float f) {
      super(i, f);
  }


  /** Gets the Enumeration of values for pair (key1, key2). */
  public   Enumeration elements (Object key1, Object key2) {
      Vector v = getVector(key1, key2);
      if (v == null) {
          v = new Vector();
      }
      return v.elements();
  }

  /** Gets the vector containing the elements for that key.*/
  public   Vector getVector (Object key1, Object key2) {
      Object val = get(key1, key2);
      if (val != null) {
          Vector v;
          try {
            v = (Vector)val;
          }
          catch (ClassCastException e) {
            v = null;
          }
            return v;
      }
      else
          return null;
  }


  /** Tests if the value is contained in any vector for any key. */
  public   boolean contains (Object key1, Object key2, Object value) {
      Vector v = getVector(key1, key2);
      if (v != null)
          return v.contains(value);
      else
          return false;
  }

  /** Adds a value to the vector of values for that key. */
  public   Object put (Object key1, Object key2, Object value) {
      Hashtable h = getHashtable (key1);
      if (h == null) {
          h = new HashtableOfVectors();  //----------->> this is central statement <<--------------
            put(key1, h);
      }
      return h.put (key2, value);
  }

  /** Removes  a value from the vector of values for that key. */
  public   Object remove (Object key1, Object key2, Object value) {
      Vector v = getVector(key1, key2);
      if (v != null) {
            v.removeElement(value);
      }
      return value;
  }

  /** Removes all elements from the vector for that key. */
  public   void removeAll (Object key1, Object key2) {
      Vector v = getVector(key1, key2);
      if (v != null) {
            v.removeAllElements();
      }
  }

}
