package mosaic.util.custom;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

/**
 * Simple Component to allow a user to select or enter a FilePath.
 * Files, Root drives and directories can be chosen.
 *
 * created and used usually by FilePathEditor
 *
 * @version 08 Apr 1999
 * @author Thomas Hilpold
 */
public class FilePathCustomEditor extends JPanel {

    /**
     * instance reports changes to editor.
     * editor must not be null.
     * */
    FilePathCustomEditor(FilePathEditor editor) {
        this.editor = editor;
        fileSysView = FileSystemView.getFileSystemView();
        setupGui();
        changeToEnteredFile(editor.getAsText());
    }

    /** creates a Gui and adds ActionListeners */
    protected void setupGui() {
        JPanel northPanel, southPanel;
        JScrollPane scroll = new JScrollPane();

        setPreferredSize(new Dimension(400, 300));
        setLayout(new BorderLayout());
        dirUpButton = new JButton("dirUp");
        //north
        northPanel = new JPanel();
        northPanel.add(dirUpButton);
        add(northPanel, BorderLayout.NORTH);
        //center
        fileAndDirList = new JList();
        scroll.getViewport().setView(fileAndDirList);
        add(scroll, BorderLayout.CENTER);
        //south
        southPanel = new JPanel();
        southPanel.setLayout(new BorderLayout());
        fileTextField = new JTextField();
        southPanel.add(fileTextField, BorderLayout.CENTER);
        bsButton = new JButton(" \\ ");
        southPanel.add(bsButton, BorderLayout.EAST);
        add(southPanel, BorderLayout.SOUTH);


        /* add Listeners */
        //To Go up one Directory
        dirUpButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (DEBUG[0]) System.err.println(" actionperformed " +e);
                    if (e.getSource() == dirUpButton)
                        if (current != null)
                            goOneUpFrom(current);
                }
        });

        //to add a backspace char to the current test in the Textfield
        bsButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (DEBUG[0]) System.err.println(" actionperformed " +e);
                    if (e.getSource() == bsButton) {
                        fileTextField.requestFocus();
                        fileTextField.setText("" + fileTextField.getText() + '\\');
                    }
                }
        });

        //to react to a user selection in the list
        //set val in propertyeditor
        //dir selected -> change to this dir
        fileAndDirList.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    if (DEBUG[0]) System.err.println(" valuechanged " +e);
                    if (e.getValueIsAdjusting())
                        return;

                    if (e.getSource() == fileAndDirList) {
                        if (fileAndDirList.isSelectionEmpty()) {
                        }
                        else {
                            int index = fileAndDirList.getSelectedIndex();
                            if ((index < curDir.length) &&
                               (index > -1)) {
                                select(curDir[index]);
                                if (curDir[index].isDirectory())
                                    changeToDir(curDir[index]);
                            }
                        }
                    }
                }
        });

        //user pressed enter in the textfield or textfield lost focus
        fileTextField.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if(e.getSource() == fileTextField) {
                        changeToEnteredFile(fileTextField.getText());
                    }
                }
        });

        fileTextField.addFocusListener(new FocusListener() {
                public void focusLost(FocusEvent e) {
                    if (e.getSource() == fileTextField) {
                        changeToEnteredFile(fileTextField.getText());
                    }
                }

                public void focusGained(FocusEvent e) {
                }
        });
    }

    /** tries to change the GUI and update the propertyeditor */
    protected void changeToEnteredFile(String afile) {
        File enteredFile;
        File parent;
        enteredFile = new File("" + afile);
        if (enteredFile.isDirectory()) {
            changeToDir(enteredFile);
            select(enteredFile);
        }
        else {
            parent = new File("" + enteredFile.getParent());
            if ((parent != null) && (parent.isDirectory())) {
                changeToDir(parent);
                select(enteredFile);
            }
            else {
                System.err.println(" no FILE in ROOT-DIR possible ");
                changeToDir(null);
                select(null);
            }
        }
    }


    /**
     * changes to the parent of the dir given
     * in case of a file the parent of its parent will
     * be shown
     * null will show the roots of the filesystem
     * e.g. on windows a:..z:
     * recursion used here !
     */
    protected void goOneUpFrom(File dirOrFile) {
        File parent;

        if (DEBUG[0]) System.err.println("oneup" + dirOrFile);
        if (dirOrFile != null) {
            parent = new File("" + dirOrFile.getParent());
            if (fileSysView.isRoot(dirOrFile)) {
                changeToDir(null);
                select(null);
            }
            else if (dirOrFile.isDirectory()) {
                changeToDir(parent);
                select(parent);
            }
            else {
                //have a file show parent of parent which is
                // a dir or null (the roots)
                goOneUpFrom(new File("" + parent));
            }
        }
        else {
            changeToDir(null);
            select(null);
        }

    }

    /**
     * sets the current directory.
     * if dir is null or no directory is given,
     * the roots of the filesystem will be shown
     */
    protected void changeToDir(File dir) {
        if (DEBUG[0]) System.err.println(" ChangetoDir " + dir);
        if (dir != null) {
            if (dir.isDirectory()) {
                //hidden files are not shown !
                curDir = fileSysView.getFiles(dir, false);
                sortDir(curDir);
                updateList(curDir);
            }
            else {
                System.err.println(" Error, not a directory ");
            }
       }
       else {
            //schould show roots
            curDir = fileSysView.getRoots();
            sortDir(curDir);
            updateList(curDir);
        }
    }

    /**
     * selects a file or directory represented
     * by a File Object. In case of a Directory the current selection will
     * be cleared.
     */
    protected void select(File fullPath) {
        if (DEBUG[0]) System.err.println(" Select " + fullPath);
        if (fullPath == null) {
            fileTextField.setText("");
            fileAndDirList.clearSelection();
            current = null;
            editor.setValue(null);
        }
        else {
            if (DEBUG[0]) {
                System.err.println(" fullPath = " + fullPath);
                System.err.println(" getPath = " + fullPath.getPath());
                System.err.println(" getAbsolutePath = " + fullPath.getAbsolutePath());
                System.err.println(" getName = " + fullPath.getName());
            }
            fileTextField.setText("" + fullPath.getAbsolutePath());
            editor.setAsText(fullPath.getAbsolutePath());
            current = fullPath;
            if (current.isDirectory())
                fileAndDirList.clearSelection();
        }
    }

    /**
     * Shows the given File[] in the JList using simple Strings.
     */
    protected void updateList(File[] dir) {
        final DefaultListModel listModel;
        if (DEBUG[0]) System.err.println(" updateList " + dir);
        if (dir != null) {
            fileAndDirList.clearSelection();
            listModel = new DefaultListModel();
            for (int i = 0; i < dir.length; i++) {
                if (fileSysView.isRoot(dir[i]))
                    listModel.addElement("<ROOT> "+ dir[i].getPath());
                else if (dir[i].isDirectory())
                    listModel.addElement("<DIR> "+ dir[i].getName());
                else
                    listModel.addElement(dir[i].getName());
            }
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    fileAndDirList.setModel(listModel);
                }
            });
        }
    }

    /**
     * sorts a dir with selection sort.
     * Has some optimizations to keep the number of
     * time consuming messages low.
     */
    public void sortDir(File[] dir) {
        int posOfLowest = 0;
        String[] dirS = new String[dir.length];
        boolean[] isDir = new boolean[dir.length];
        boolean lowestIsDir, swapB;
        String lowestS, swapS;
        File lowest;
        File swap;
        // prepare arrays (optimization)
        for (int i = 0; i < dirS.length; i++) {
            dirS[i] = dir[i].getName().toLowerCase();
            isDir[i] = dir[i].isDirectory();
        }
        //perform selection sort
        for(int i = 0; i < dir.length - 1; i++) {
            lowest = dir[i];
            lowestS = dirS[i];
            lowestIsDir = isDir[i];
            posOfLowest = i;
            for (int j = i + 1; j < dir.length; j++) {
                boolean curIsDir = isDir[j];
                if (((dirS[j].compareTo(lowestS) < 0)
                     && (curIsDir == lowestIsDir)
                    ) || ((curIsDir) && (! lowestIsDir))){
                    posOfLowest = j;
                    lowest = dir[j];
                    lowestS = dirS[j];
                    lowestIsDir = isDir[j];
                    }
            }
            // smallest from i+1 to dir.length found, swap it with i
            swap = dir[i];
            dir[i] = lowest;
            dir[posOfLowest] = swap;

            swapS = dirS[i];
            dirS[i] = lowestS;
            dirS[posOfLowest] = swapS;

            swapB = isDir[i];
            isDir[i] = isDir[posOfLowest];
            isDir[posOfLowest] = swapB;
        }
    }

    //
    // Data
    //
    protected FilePathEditor editor;
    protected FileSystemView fileSysView;
    protected File current; //the selected File or DIRECTORY
    protected File[] curDir;
    protected JButton dirUpButton;
    protected JButton bsButton; //for adding a backslash
    protected JList fileAndDirList;
    protected JTextField fileTextField;

    //
    // Debugging output
    //
    static boolean[] DEBUG = new boolean[] {false};
}
