package mosaic.vis.input;

import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;
import mosaic.vis.input.ui.*;

/**
* Animation view to visualize and allow input of numeric values.
* Can serve as variable change listeners for numeric variables.
* Can serve as userInputSource for numeric values
* <p>
* Use the view by adding it as VariableChangeListener to a numeric Variable (e.g. IntVariable or DoubleVariable).
*
* <b>Costumization: </b>
* <ul>
* <li> set the label by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
* @author Bernhard Reitinger (redesigned !) 08/03/2000
* @version %I% %G%
*/

public class SliderCondInputView extends BasicNumberInputView {
    JSlider jSlider = new JSlider();
    BorderLayout borderLayout1 = new BorderLayout();
    private SliderCondInputUI sliderUI = new SliderCondInputUI(jSlider,this);
    private int precision = 2;

    /** Default constructor */
    public SliderCondInputView() {
        super();

        try  {
 	    jSlider.setUI(sliderUI);
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
	this.setLayout(borderLayout1);
        jSlider.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                //if ( ! ((JSlider)e.getSource()).getValueIsAdjusting() ) {
                    setValue(new Double(sliderUI.getValueStr()));
		            triggerEvent();
                //}
            }
	    });
 	this.add(jSlider);
	sliderUI.setPrecision(precision);
	jSlider.setMinimum((int)(getMinimum()*Math.pow((double)10,(double)precision)));
	jSlider.setMaximum((int)(getMaximum()*Math.pow((double)10,(double)precision)));
	jSlider.setMajorTickSpacing(((int)(getMajorTickSpacing()*Math.pow((double)10,(double)precision))));
	jSlider.setMinorTickSpacing((int)(getMinorTickSpacing()*Math.pow((double)10,(double)precision)));
    }

    // Overwrite paintGfx to avoid painting of background rectangle and
    // label
    public void paintWithGfx(Graphics g) {
    }

    // returns slider
    public JSlider getSlider() { return jSlider; }

    // Properties forwarded to jslider
    public void setExtent(int ext)         { jSlider.setExtent(ext);           }

    public int getExtent()                 { return jSlider.getExtent();       }

    public void setMajorTickSpacing(int n) {
	jSlider.setMajorTickSpacing(((int)(n*Math.pow((double)10,(double)precision))));
	jSlider.setValue((int)(jSlider.getValue()*Math.pow((double)10,(double)precision)));
	sliderUI.setPrecision(precision);
    }

    public int getMajorTickSpacing() {
	return (int)(jSlider.getMajorTickSpacing()/Math.pow((double)10,(double)precision));
    }

    public void setMaximum(int max) { 
	jSlider.setMaximum((int)(max*Math.pow((double)10,(double)precision)));
	jSlider.setValue((int)(jSlider.getValue()*Math.pow((double)10,(double)precision)));
	sliderUI.setPrecision(precision);
    }

    public int getMaximum() {
	return (int)(jSlider.getMaximum()/Math.pow((double)10,(double)precision));
    }

    public void setMinimum(int min) {
	jSlider.setMinimum((int)(min*Math.pow((double)10,(double)precision)));
	jSlider.setValue((int)(jSlider.getValue()*Math.pow((double)10,(double)precision)));
	sliderUI.setPrecision(precision);
    }

    public int getMinimum() { 
	return (int)(jSlider.getMinimum()/Math.pow((double)10,(double)precision));
    }

    public void setMinorTickSpacing(int n) { 
	jSlider.setMinorTickSpacing((int)(n*Math.pow((double)10,(double)precision)));
	jSlider.setValue((int)(jSlider.getValue()*Math.pow((double)10,(double)precision)));
	sliderUI.setPrecision(precision);
    }

    public int getMinorTickSpacing() { 
	return (int)(jSlider.getMinorTickSpacing()/Math.pow((double)10,(double)precision));
    }

    public void setOrientation(int o)      { jSlider.setOrientation(o);        }

    public int getOrientation()            { return jSlider.getOrientation();  }

    public void setPaintLabels(boolean b)  { jSlider.setPaintLabels(b);        }

    public boolean getPaintLabels()        { return jSlider.getPaintLabels();  }

    public void setPaintTicks(boolean b)   { jSlider.setPaintTicks(b);         }

    public boolean getPaintTicks()         { return jSlider.getPaintTicks();   }

    public void setPaintTrack(boolean b)   { jSlider.setPaintTrack(b);         }

    public boolean getPaintTrack()         { return jSlider.getPaintTrack();   }

    public void setSnapToTicks(boolean b)  { jSlider.setSnapToTicks(b);        }

    public boolean getSnapToTicks()        { return jSlider.getSnapToTicks();  }

    public void setInitValue(int val) {
    	jSlider.setValue((int)(val*Math.pow((double)10,(double)precision)));
        setValue(new Double(val));
        triggerEvent();
    }

    public int getInitValue() { 
	return (int)(jSlider.getValue()/Math.pow((double)10,(double)precision));
    }

    public void setPrecision(int val) { 
	precision = val;
	setMinorTickSpacing(getMinorTickSpacing());
	setMajorTickSpacing(getMajorTickSpacing());
	setMinimum(getMinimum());
	setMaximum(getMaximum());
	jSlider.setValue((int)(jSlider.getValue()*Math.pow((double)10,(double)precision)));
	sliderUI.setPrecision(precision);
    }

    public int getPrecision() {
	return precision;
    }


    public static void main(String[] args) {
	javax.swing.JFrame f = new javax.swing.JFrame();
  	SliderCondInputView s = new SliderCondInputView();
// 	JSlider s = new JSlider();
	javax.swing.JPanel panel = new javax.swing.JPanel();

// 	System.out.println(s.getUI());
	panel.setLayout(null);
	s.setMajorTickSpacing(100);
	s.setPaintTicks(true);
	s.setLocation(0,0);
// 	s.setMinimum(-10);
 	s.setMaximum(100);
	s.setPrecision(2);
//  	s.setInitValue(50);
	
	s.setBounds(30,0,200,100);
	panel.add(s);
	f.getContentPane().add(s);
	f.setSize(500,100);
	f.setLocation(100,100);
	f.show();
	
    }


}
