
//Project:      SimBeans kernel
//Version:      0.3
//Copyright:    Copyright (c) 1998
//Author:       Herbert Praehofer, Johannes Sametinger, Alois Stritzinger
//Affiliation:  Johannes Kepler University, Linz, Austria

// -----------------------------------------------------------------------------------------------------------
// class VectorTextView     ----------------------------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------

package mosaic.vis.output;

import java.awt.*;
import java.util.*;

/**
* Animation view to visualize a vector of Objects.
* <p>
* Use the view by adding it as VariableChangeListener to a <code>VectorVariable</code>.
* Displays the textual representation of the Objects using toString.  
* <p>
* <b>Costumization: </b>
* <ul>
* <li> set the label of the view by <code>setLabel</code>
* <li> set background color by <code>setBackground</code>
* <li> set foreground color by <code>setForeground</code>
* </ul>
*
* @author Herbert Praehofer
* @author Johannes Sametinger
* @author Alois Stritzinger
* @version %I% %G%
*/
public class VectorTextView extends TextView {

  /** Default constructor QueueView */
  public VectorTextView() {
  }

  /**
  * Reacts to a new property value by setting the Vector value.
  * Sets the text by calling <code>toString</code> of the vector.
  * Called by super.propertyChange.
  * @param newVal The new vector object received from the property change
  */
  public void setValue (Object newVal) {
      if ((newVal instanceof Vector) || (newVal == null)) {
          super.setValue(newVal);
          String htext = "";
          if (getValue() != null) {
              Vector v = (Vector)getValue();
              for (int i = 0; i < v.size(); i++) {
                  htext = htext + v.elementAt(i).toString();
              }
          setText ( htext );
          }
      }
      else
            System.out.println(">> tried to send non Vector value to " + toString());
  }

}

